--[[
@ by Ziuta
@ 27.02.2015
@ v1.0.0
@ edit: 28.06.2015
	v1.1.0: added the ability to load your own texts help
			added the ability to load button
@ edit2: 08.07.2015
	v1.2.0: added the ability to show/hide help text (F1)
			added the ability to run animations of the vehicle and on approach
@ edit3: 04.05.2016
	v1.3.0: added the initial state (time) animation
			rewriting load/saveAttributes
@ edit4: 20.05.2016
	v1.3.1: added the ability to not save the state of animation
@ edit5: 20.09.2016
	v1.3.5: added protection to load a smaller number of states of the number of animations
			added force updates objects <movingParts>
			
@ convert to FS17: 04.11.2016

http://landwirtschafts-simulator.pl/
Thanks for your help Koper, lip4 and Decker_MMIV

You can use and edit but keep the original author ;)
]]

--[[
modDesc
<specialization name="animXML" className="animXML" filename="animXML.lua"/>


...
<specialization name="animatedVehicle" />
<specialization name="cylindered" />
...
<specialization name="animXML" /> 

XML-file
<animate>
	<object index="0>0|6" distance="2" startTime="0" animationName="moveSupport" input="support" save="true" renderText="true/false" textP="supportUp" textN="supportDown" together="true/false" onApproach="true/false" />
</animate>

INFO:
index - object, relative to which the distance is measured
distance - min distance in meters
startTime -  the initial state (time) animation: 0 or 1
animationName - the name of the animation from an XML file
input - the name of a button loaded with modDesc
save - if you do not want the animation save during the game save set "false", otherwise set to "true"
renderText - displaying the text in the help window (F1) if "true"
textP/textN - text displayed in the help window loaded with modDesc (eg. P- open door, N- close door)
together - the ability to run animations of the vehicle and on approach
onApproach - if "true" animation runs only on the approach, if "false" animation runs only from the vehicle
]]

animXML = {}

function animXML.prerequisitesPresent(specializations)
   return SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations) and SpecializationUtil.hasSpecialization(Cylindered, specializations)
end

function animXML:load(savegame)	
	self.getIsInRange = animXML.getIsInRange
	self.setAnim = animXML.setAnim
	self.onApproach = {}
	local i = 0
    while true do
		local key = string.format("vehicle.animate.object(%d)", i)
		if not hasXMLProperty(self.xmlFile, key) then
            break
        end
		local ob = {}
		ob.index = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key.."#index"))
		ob.animationName = getXMLString(self.xmlFile, key.."#animationName")
		ob.minDistance = Utils.getNoNil(getXMLFloat(self.xmlFile, key.."#distance"), 2.0)
		ob.startTime = Utils.getNoNil(getXMLFloat(self.xmlFile, key.."#startTime"), 0)
		if ob.startTime == 0 then
			ob.animIsChange = false
		elseif ob.startTime == 1 then
			ob.animIsChange = true
		end			
		ob.input = getXMLString(self.xmlFile, key.."#input")
		ob.save = Utils.getNoNil(getXMLBool(self.xmlFile, key.."#save"), true)
		local text = Utils.getNoNil(getXMLBool(self.xmlFile, key.."#renderText"), true)
		if text then
			ob.text = text
			ob.textP = getXMLString(self.xmlFile, key.."#textP")
			ob.textN = getXMLString(self.xmlFile, key.."#textN")
		end
		local together = Utils.getNoNil(getXMLBool(self.xmlFile, key.."#together"), false)
		if together then
			ob.together = together
		else
			ob.onApp = Utils.getNoNil(getXMLBool(self.xmlFile, key.."#onApproach"), true)
		end
		table.insert(self.onApproach, ob)
		i = i + 1
	end
	
	if savegame ~= nil then
		local animTime = getXMLString(savegame.xmlFile, savegame.key.."#animTime")	
		if animTime ~= nil then	
			local tm = Utils.splitString(" ", animTime)
			for i=1, table.getn(self.onApproach) do
				if (table.getn(tm)-1) == table.getn(self.onApproach) then
					if self.onApproach[i].save then
						self:setAnimationTime(self.onApproach[i].animationName,tm[i],true)
						if tm[i] == "0" then
							self.onApproach[i].animIsChange = false
						elseif tm[i] == "1" then
							self.onApproach[i].animIsChange = true
						end
					else
						self:setAnimationTime(self.onApproach[i].animationName,self.onApproach[i].startTime,true)
					end
				else
					self:setAnimationTime(self.onApproach[i].animationName,self.onApproach[i].startTime,true)
				end
			end
		elseif animTime == nil then
			for i=1, table.getn(self.onApproach) do
				self:setAnimationTime(self.onApproach[i].animationName,self.onApproach[i].startTime,true)
			end
		end
	end
end

function animXML:delete()
end

function animXML:readStream(streamId, connection)
	for i=1, table.getn(self.onApproach) do
		local state = streamReadBool(streamId)
		self:setAnim(i, state, true)
	end
end

function animXML:writeStream(streamId, connection)	
	for i=1, table.getn(self.onApproach) do
		local ob = self.onApproach[i]
		local state = ob.animIsChange
		streamWriteBool(streamId, state)		
	end
end

function animXML:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'animTime="'
	for i=1, table.getn(self.onApproach) do
		local ob = self.onApproach[i]
		local aTime = self:getAnimationTime(ob.animationName)
		attributes = attributes..tostring(aTime).." "
	end
	attributes = attributes..'"'
	return attributes
end

function animXML:mouseEvent(posX, posY, isDown, isUp, button)
end

function animXML:keyEvent(unicode, sym, modifier, isDown)
end

function animXML:update(dt)
	if self.isClient then
		for i=1, table.getn(self.onApproach) do
			if self.onApproach[i].together then
				if self:getIsInRange(i) then
					if InputBinding.hasEvent(InputBinding[self.onApproach[i].input]) then
						self:setAnim(i, not self.onApproach[i].animIsChange)
					end
					if self.onApproach[i].text then
						if self.onApproach[i].animIsChange then
							g_currentMission:addHelpButtonText(g_i18n:getText(self.onApproach[i].textN), InputBinding[self.onApproach[i].input])
						else
							g_currentMission:addHelpButtonText(g_i18n:getText(self.onApproach[i].textP), InputBinding[self.onApproach[i].input])
						end
					end
				elseif self:getIsActiveForInput(true) then
					if InputBinding.hasEvent(InputBinding[self.onApproach[i].input]) then
						self:setAnim(i, not self.onApproach[i].animIsChange)
					end
					if self.onApproach[i].text then
						if self.onApproach[i].animIsChange then
							g_currentMission:addHelpButtonText(g_i18n:getText(self.onApproach[i].textN), InputBinding[self.onApproach[i].input])
						else
							g_currentMission:addHelpButtonText(g_i18n:getText(self.onApproach[i].textP), InputBinding[self.onApproach[i].input])
						end
					end
				end				
			else
				if self.onApproach[i].onApp then
					if self:getIsInRange(i) then
						if InputBinding.hasEvent(InputBinding[self.onApproach[i].input]) then
							self:setAnim(i, not self.onApproach[i].animIsChange)
						end
						if self.onApproach[i].text then
							if self.onApproach[i].animIsChange then
								g_currentMission:addHelpButtonText(g_i18n:getText(self.onApproach[i].textN), InputBinding[self.onApproach[i].input])
							else
								g_currentMission:addHelpButtonText(g_i18n:getText(self.onApproach[i].textP), InputBinding[self.onApproach[i].input])
							end
						end
					end
				elseif not self.onApproach[i].onApp and self:getIsActiveForInput(true) then
					if InputBinding.hasEvent(InputBinding[self.onApproach[i].input]) then
						self:setAnim(i, not self.onApproach[i].animIsChange)
					end
					if self.onApproach[i].text then
						if self.onApproach[i].animIsChange then
							g_currentMission:addHelpButtonText(g_i18n:getText(self.onApproach[i].textN), InputBinding[self.onApproach[i].input])
						else
							g_currentMission:addHelpButtonText(g_i18n:getText(self.onApproach[i].textP), InputBinding[self.onApproach[i].input])
						end
					end
				end
			end
		end
	end
end

function animXML:updateTick(dt)
	for _, part in pairs(self.activeDirtyMovingParts) do
		Cylindered.setDirty(self, part);
	end
end

function animXML:draw()
end

function animXML:getIsInRange(i)
	if not self.isEntered then
		local rx, ry, rz = getWorldTranslation(g_currentMission.player.rootNode)
		local ox, oy, oz = getWorldTranslation(self.onApproach[i].index)
		if rx == nil or ry == nil or rz == nil or oy == nil or oz == nil then
			return false
		else
			local distance = Utils.vector3Length(rx-ox, ry-oy, rz-oz)
			if distance <= self.onApproach[i].minDistance then
				return true
			else
				return false
			end
		end
	else
		return false
	end
end

function animXML:setAnim(index, state, noEventSend)
	setAnimEvent.sendEvent(self, index, state, noEventSend)
	local ob = self.onApproach[index]
	local animTime = self:getAnimationTime(ob.animationName)
	if state and animTime < 1 then
		self:playAnimation(ob.animationName,1,animTime,true)
	elseif animTime > 0 then
		self:playAnimation(ob.animationName,-1,animTime,true)
	end
	for a=1, table.getn(self.movingParts) do
		Cylindered.updateMovingPart(self, self.movingParts[a], false);
	end
	ob.animIsChange = state
end

--
-- MP ready
--

setAnimEvent = {}
setAnimEvent_mt = Class(setAnimEvent, Event)

InitEventClass(setAnimEvent, "setAnimEvent")

function setAnimEvent:emptyNew()
    local self = Event:new(setAnimEvent_mt)
    return self
end

function setAnimEvent:new(object, index, state)
    local self = setAnimEvent:emptyNew()
    self.object = object
    self.index = index
	self.state = state
    return self
end

function setAnimEvent:readStream(streamId, connection)	
	self.object = readNetworkNodeObject(streamId)	
	self.index = streamReadInt8(streamId)
	self.state = streamReadBool(streamId)	
    self:run(connection)
end

function setAnimEvent:writeStream(streamId, connection)
    writeNetworkNodeObject(streamId, self.object)
	streamWriteInt8(streamId, self.index)
	streamWriteBool(streamId, self.state)
end;

function setAnimEvent:run(connection)
    self.object:setAnim(self.index, self.state, true)
    if not connection:getIsServer() then
        g_server:broadcastEvent(setAnimEvent:new(self.object, self.index, self.state), nil, connection, self.object)
    end
end

function setAnimEvent.sendEvent(vehicle, index, state, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(setAnimEvent:new(vehicle, index, state), nil, nil, vehicle)
		else
			g_client:getServerConnection():sendEvent(setAnimEvent:new(vehicle, index, state))
		end
	end
end